<?php

namespace Drupal\tweet_reference;

use Drupal\Core\Cache\Cache;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Language\LanguageInterface;

/**
 * Class TweetSync.
 */
class TweetSync {

  /**
   * The configuration object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The tweet storage service.
   *
   * @var \Drupal\tweet_reference\TweetStorage
   */
  protected $tweetStorage;

  /**
   * The tweet storage service.
   *
   * @var \Drupal\tweet_reference\TwitterProvider
   */
  protected $twitterProvider;

  /**
   * Constructs a new TweetStorage object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\tweet_reference\TweetStorage $tweet_storage
   *   The tweet storage service.
   * @param \Drupal\tweet_reference\TwitterProvider $twitter_provider
   *   The tweet storage service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TweetStorage $tweet_storage, TwitterProvider $twitter_provider) {
    $this->config = $config_factory->get('tweet_reference.twitter_api');
    $this->tweetStorage = $tweet_storage;
    $this->twitterProvider = $twitter_provider;
  }

  /**
   * Updates tweets and other data by cron.
   */
  public function cronWorker() {
    $lifetime = intval($this->config->get('cache_lifetime'));
    if (empty($lifetime)) {
      $lifetime = 172800;
    }

    $current_time = time();

    $tweets = $this->tweetStorage->loadOutdatedTweetRecords($current_time - $lifetime, 80);

    if (empty($tweets)) {
      return;
    }

    $tweet_ids = [];

    foreach ($tweets as $tweet) {
      $tweet_ids[$tweet['tweet_id']] = $tweet['tweet_id'];
    }

    $this->fetchAndUpdateTweets($tweet_ids, $current_time);
  }

  /**
   * Updates single tweet.
   */
  public function syncSingleTweet($tweet_id) {
    $this->fetchAndUpdateTweets([$tweet_id], time());
  }

  /**
   * Fetches and updates tweets.
   */
  protected function fetchAndUpdateTweets($tweet_ids, $current_time) {
    $remote_tweets = $this->twitterProvider->getTweets($this->config->get('twitter_key'), $this->config->get('twitter_secret'), $tweet_ids);

    if (is_null($remote_tweets)) {
      return;
    }

    $remote_tweet_ids = [];
    $cache_tags = [];

    foreach ($remote_tweets as $remote_tweet) {
      $tweet_id = $remote_tweet['id_str'];
      $remote_tweet_ids[$tweet_id] = $tweet_id;
      $this->updateTweetsByTweetId($tweet_id, $remote_tweet, $current_time, $cache_tags);
    }

    foreach ($tweet_ids as $tweet_id) {
      if (!isset($remote_tweet_ids[$tweet_id])) {
        $this->deleteTweetsByTweetId($tweet_id, $cache_tags);
      }
    }

    Cache::invalidateTags(array_values($cache_tags));
  }

  /**
   * Updates tweets having common tweet identifier.
   */
  protected function updateTweetsByTweetId($tweet_id, $remote_tweet, $current_time, &$cache_tags) {
    $tweets = $this->tweetStorage->loadTweetRecordsByTweetId($tweet_id);
    $remote_tweet_data = $remote_tweet;

    foreach ($tweets as $tweet) {
      $tweet_data = $tweet['tweet_data'];

      $tweet['synced_time'] = $current_time;
      $tweet['langcode'] = $remote_tweet['lang'];
      $tweet['tweet_time'] = strtotime($remote_tweet['created_at']);
      $tweet['user_id'] = $remote_tweet['user']['id_str'];
      $tweet['tweet_data'] = $remote_tweet;

      $this->tweetStorage->saveTweetRecord($tweet);

      // @todo Do not clear caches if not necessary.
      // Calculate $checksum = Crypt::hashBase64(serialize($data));
      $tag = 'tweet_reference:' . $tweet['id'];
      $cache_tags[$tag] = $tag;

      if (!empty($tweet['entity_type']) && !empty($tweet['entity_id'])) {
        $tag = $tweet['entity_type'] . ':' . $tweet['entity_id'];
        $cache_tags[$tag] = $tag;
      }
    }
  }

  /**
   * Deletes tweets having common tweet identifier.
   */
  protected function deleteTweetsByTweetId($tweet_id, &$cache_tags) {
    $tweets = $this->tweetStorage->loadTweetRecordsByTweetId($tweet_id);

    foreach ($tweets as $tweet) {
      $this->tweetStorage->deleteTweetRecord($tweet['id']);

      $tag = 'tweet_reference:' . $tweet['id'];
      $cache_tags[$tag] = $tag;

      if (!empty($tweet['entity_type']) && !empty($tweet['entity_id'])) {
        $tag = $tweet['entity_type'] . ':' . $tweet['entity_id'];
        $cache_tags[$tag] = $tag;
      }
    }
  }

  /**
   * Returns the current content language.
   */
  public static function getContentLanguage() {
    static $content_langcode = NULL;

    if (empty($content_langcode)) {
      $content_langcode = \Drupal::languageManager()->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)->getId();
    }

    return $content_langcode;
  }

}
